﻿Imports System.Security.Principal
Imports System.Configuration
Imports System.Web.Configuration
Imports System.Collections.Generic
Imports MFA.SecureAuth.Resource
Imports System.IO
Imports System.Net.Configuration
Imports MFA.Utilities
Imports Net.SecureAuth.Transaction
Imports System.Web.Security
Imports Ganss.XSS

Public Class FormPost
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Response.Expires = 0
        Response.ExpiresAbsolute = Now().Subtract(New TimeSpan(1, 0, 0, 0))
        Response.CacheControl = "no-cache"

        Session("PreAuthPage") = "FormPostCustom.aspx"

        Try
            HttpContext.Current.Session("UserLanguage") = HttpContext.Current.Request.UserLanguages(0)
        Catch
            '' if no Language default in client's browser, default to US English
            HttpContext.Current.Session("UserLanguage") = "en-US"
        End Try
        Dim sanitizer As HtmlSanitizer = New HtmlSanitizer()
        '' This is how external apps can Form Post a User ID / Password to us
        Dim user As String = Request.Form("UserID")
        Dim password As String = Request.Form("Password")
        Dim mode As String = Request.Form("Mode")
        Dim SharedSecret As String = Request.Form("SharedSecret")
		If user = "" Then
			'' Redirect to restart.aspx or just produce a 404
			Logging.LogExtendedMsg(Logging.LogEnum.PreAuth + 3100, 3, user, "No UserID supplied in Custom Form Post.")
			''Response.Redirect("~/restart.aspx", True)
			Response.Clear()
			Response.StatusCode = 404
			Response.Status = "404 Page Not Found"
			Response.End()
		End If
        Logging.LogExtendedMsg(Logging.LogEnum.PreAuth + 3100, 3, user, "The current identity from Form Post: " & user)
		if mode = "" Then
			mode = "PUBLIC"
			Logging.LogExtendedMsg(Logging.LogEnum.PreAuth + 3100, 3, user, "No Mode supplied in Custom Form Post, defaulting to PUBLIC")
		End If
		If Request("Debug") <> "" Then
            Response.Write("User: " & sanitizer.Sanitize(user) & "<BR/>")
            Response.Write("Mode: " & sanitizer.Sanitize(mode) & "<BR/>")
            Response.Write("Posted Secret: " & sanitizer.Sanitize(SharedSecret) & "<BR/>")
            Response.Write("Get Shared Secret: " & ConfigurationManager.AppSettings("GetSharedSecret").ToString() & "<BR/>")
            Response.Write("IP Address: " & Tools.GetClientHostAddress() & "<BR/>")
            If Membership.ValidateUser(user, password) Then
                Response.Write("UserValidated: True")
            Else
                Response.Write("UserValidated: False")
            End If
            Response.End()
        End If

        '' This is how external apps can Form Post a Private/Public selection to us
        Dim publicprivatecookie As New HttpCookie("SecureAuthLogin" & HttpContext.Current.Request.ApplicationPath.Substring(1))
        publicprivatecookie.Expires = DateTime.Now.AddYears(5)
        publicprivatecookie.HttpOnly = True
        publicprivatecookie.Value = mode.ToUpper()  '' PUBLIC or PRIVATE
#If Not Debug Then
        publicprivatecookie.Secure = FormsAuthentication.RequireSSL
#End If

        Dim UserValidated As Boolean = False
        '' 0 = No User check, 1 = GetUser, 2 = GetUser + SharedSecret, 3 = ValidateUser, 4 = ValidateUser + SharedSecret
        If ConfigurationManager.AppSettings("FormPostValidationMode") = 1 AndAlso user <> "" AndAlso Not Membership.GetUser(user) Is Nothing Then
            UserValidated = True
        ElseIf ConfigurationManager.AppSettings("FormPostValidationMode") = 2 AndAlso user <> "" AndAlso Not Membership.GetUser(user) Is Nothing AndAlso SharedSecret.ToUpper() = ConfigurationManager.AppSettings("GetSharedSecret").ToString().ToUpper() Then
            UserValidated = True
        ElseIf ConfigurationManager.AppSettings("FormPostValidationMode") = 3 AndAlso user <> "" AndAlso password <> "" AndAlso Membership.ValidateUser(user, password) Then
            UserValidated = True
        ElseIf ConfigurationManager.AppSettings("FormPostValidationMode") = 4 AndAlso user <> "" AndAlso password <> "" AndAlso Membership.ValidateUser(user, password) AndAlso SharedSecret.ToUpper() = ConfigurationManager.AppSettings("GetSharedSecret").ToString().ToUpper() Then
            UserValidated = True
        ElseIf ConfigurationManager.AppSettings("FormPostValidationMode") = 0 Then
            UserValidated = True
        End If

        If UserValidated Then
            Response.AppendCookie(publicprivatecookie)
            Response.AppendCookie(Encryption.CreateFormsTicket(ConfigurationManager.AppSettings("GetSharedSecret").ToString(), user, FormsAuthentication.Timeout.TotalMinutes, user, ConfigurationManager.AppSettings("PreAuthCookieName")))
            HttpContext.Current.Session("SendID") = user

            Logging.LogExtendedMsg(Logging.LogEnum.PreAuth + 3190, 3, user, "Custom Form Post succeeds for user: " & user)
            Dim QueryStringParams = Request.QueryString.ToString().Trim()
            Response.Redirect("~/SecureAuth.aspx" + If(QueryStringParams Is "", "", "?" + QueryStringParams), True)
		Else 
			'' Redirect to restart.aspx or just produce a 404
			Logging.LogExtendedMsg(Logging.LogEnum.PreAuth + 3190, 3, user, "Custom Form Post failed for user: " & user)
			''Response.Redirect("~/restart.aspx", True)
			Response.Clear()
			Response.StatusCode = 404
			Response.Status = "404 Page Not Found"
			Response.End()
		End If
    End Sub

End Class